#!/usr/bin/perl -w

# Leginon pause client
# WVN 4/11/06

use strict;
use sigtrap;

use legpause::netrequests qw(:ALL);

use Socket;  # or IO::Socket
use IO::Socket::INET;

# include Windows speciffic modules
use Win32;
use Win32::Process;

# include the testing library
# we import ALL exported symbols for convenience
# is wiser though to import just what you need in 
# production code 
use Win32::GuiTest qw(:ALL);

sub stopPump();
sub getTimeRemaining();
sub click_on_middle_of_window();
sub click_at_end_of_window();
sub send_large_time_to_timer_text_box();
sub bring_window_to_front();

my $socket;
my $remote_host;
my $remote_port=60000;
my $state="unpaused";
my $answer;
my $prefillTime;
my $postfillTime;

my $startTime;
my $stopTime;
my $endTime;
my $endTimeSecs;
my $endTimeMins;
my $endTimeHours;

# how often (pump) clock is checked in seconds
my $pollTime=10;

my $nargs=scalar(@ARGV);

if($nargs != 4){
  print "Usage: lpclient.pl server-host prefill-time-mins postfill-time-mins stop-time-mins\n";
  exit -1;
}

$remote_host=$ARGV[0];
$prefillTime=$ARGV[1];
$postfillTime=$ARGV[2];
$stopTime=$ARGV[3];

print "Server is on host ", $remote_host, "\n";
print "Pre-fill time is ", $prefillTime, " minutes\n";
print "Post-fill time is ", $postfillTime, " minutes\n";
print "Pump stop time is ", $stopTime, " minutes\n";

$startTime=time();
$endTime=$startTime+60*$stopTime;
($endTimeSecs, $endTimeMins, $endTimeHours) = localtime($endTime);

print "Pump will stop at ", $endTimeHours, ":", $endTimeMins, ":", $endTimeSecs, "\n\n";


# list of windows; we use this to keep all found windows
# returned by FindWindowLike()
my @windows;

# Timer textbox - where new (long) time will be entered
my @timerTextBox;

# Timer window
my @timer;

# Set button
my @button;

# search for windows matching the title "Norhof Monitor"
@windows = FindWindowLike( undef, "Norhof Monitor", "" );

# print the number of windows found (it should be one)
print "* Number of Norhof Monitor windows found: "
  . ( $#windows + 1 ) . "\n";
if ( $#windows > 0 ) {
    print
"* The Norhof Monitor program has been started more than once!\n";
    print
"* Please close excess instances of the Norhof Monitor program\n";
    print "  then re-run this program\n";
    exit 2;
}
elsif ( $#windows < 0 ) {
    print "* The program hasn't been started!\n";
    exit 1;
}

# find the Timer window
@timer =
  FindWindowLike( $windows[0], "Timer", "" );

# find the timer TextBox's
@timerTextBox =
  FindWindowLike( $timer[0], "", "^ThunderRT6TextBox" );

my $nboxes = scalar(@timerTextBox);

# WVN use the second one
# ideally we want to query them in some way to ensure we get the read-only one
# however this is not really possible as the "read-only" textbox allows text input
# only resets it after input (!)
my $timeRemainingBox = $timerTextBox[1];

# Main program loop
while(time() <= $endTime){

  if($state eq "unpaused"){
    # check if time on clock is equal to or less than pre-fill time
    # if it is then pause Leginon and then set state to paused
    if(getTimeRemaining() <= $prefillTime){
      netrequests::requestPause($remote_host, $remote_port);
      $state="paused";
      # WVN 12/6/07 - bug - problem was that the code would exit this block
      # and then go straight into the "paused" case for this if-then-else statement
      # and fail to wait for the prefill time.  So below sleep call was added.
      sleep(60*$prefillTime); # This may wait for at most a minute too long
                              # Usually this extra minute should be at least covered by the
                              # actual fill time.
                              # (getTimeRemaining() gives the time remaining rounded
                              # down to the minute which is too little time which is worse
                              # because of the resulting extra pause/ unpause that can
                              # happen with extra wait for the post-fill.) 
 
    }
  }
  elsif($state eq "paused"){
    # check if time on clock is still zero
    # if not then wait for post-fill time by sleeping,
    # unpause Leginon and then set state to unpaused
    if(getTimeRemaining() != 0){
      sleep(60*$postfillTime);
      netrequests::requestUnpause($remote_host,$remote_port);
      $state="unpaused";
    }
  }
  else{
    print "Error: somehow got bad state";
    exit -2;
  }

  sleep($pollTime);
}

netrequests::requestStop($remote_host,$remote_port);
stopPump();

exit 0;

sub getTimeRemaining(){
  my $timeRemaining;

# WVN GetWindowText() appears to get default/ starting value only
my $text = WMGetText($timeRemainingBox);

my ($day, $time, $hours, $mins);

($day, $time) = split(/ - /, $text);

($hours, $mins) = split(/:/, $time);

$timeRemaining = 24*60*$day+60*$hours+$mins;

  return $timeRemaining;
}

# this function tries to bring a window to front
#
# it takes as a parameter the window ID to bring to front
sub bring_window_to_front {
    my $window  = shift;
    my $success = 1;

    if ( SetActiveWindow($window) ) {
        print
"* Successfuly set the window id: $window active\n";
    }
    else {
        print
          "* Could not set the window id: $window active\n";
        $success = 0;
    }
    if ( SetForegroundWindow($window) ) {
        print
          "* Window id: $window brought to foreground\n";
    }
    else {
        print
"* Window id: $window could not be brought to foreground\n";
        $success = 0;
    }

    return $success;
}

# this function takes a window handle as an argument
# then it moves the mouse over the middle of the window
# and left clicks on it
#
# make sure that the container window *has* focus before
# left clicking on it, otherwise you may have surprises if
# there is another window to front that is over the 
# position where you will click
sub click_on_middle_of_window {

    # the window handle of the window we will click on
    my $window = shift;

    print
      "* Moving the mouse over the window id: $window\n";

    # get the coordinates of the window
    my ( $left, $top, $right, $bottom ) =
      GetWindowRect($window);

    # now we move the mouse right in the middle of the 
    # window
    MouseMoveAbsPix( ( $right + $left ) / 2,
        ( $top + $bottom ) / 2 );

    # ;-)
    sleep(1);

    # send LEFT CLICK event
    print "* Left Clicking on the window id: $window\n";
    SendMouse("{LeftClick}");
    sleep(1);
}

# make sure that the container window *has* focus before
# left clicking on it, otherwise you may have surprises if
# there is another window to front that is over the 
# position where you will click
sub click_at_end_of_window {
    my $offset = 8; # ad hoc offset, possibly resolution dependent
                    # to make sure we successfully click in the
                    # input area

    # the window handle of the window we will click on
    my $window = shift;

    print
      "* Moving the mouse over the window id: $window\n";

    # get the coordinates of the window
    my ( $left, $top, $right, $bottom ) =
      GetWindowRect($window);

    # now we move the mouse right in the middle of the 
    # window
    MouseMoveAbsPix(  ($right-$offset) ,
        ( $top + $bottom ) / 2 );

    # ;-)
    sleep(1);

    # send LEFT CLICK event
    print "* Left Clicking on the window id: $window\n";
    SendMouse("{LeftClick}");
    sleep(1);
}


# this function just "prints" some text in the edit window
sub send_large_time_to_timer_text_box {

    # the window ID
    my $window = shift;

    # text that will be sent to the timerTextBox

    # delete existing time and insert text for 9 days
    # my $text = "9 - 00:00";
    my $text = "\x08\x08\x08\x08\x08\x08\x08\x08\x089 - 00:00";
    
    my @text;

    # send key by key ....
    map { SendKeys($_) } split //, $text;

    # done
}


sub stopPump() {
  # get focus in timerTextBox and pick correct insertion point for text
  # WVN possibly unsafe - just picks the first timerTextBox
  click_at_end_of_window( $timerTextBox[0] );

  # now that we have focus not on the container itself,
  # but on the timerTextBox window, we can send it some text
  send_large_time_to_timer_text_box( $timerTextBox[0] );

  # find and press the Norhof Monitor Set button so the repeat time for LN2 pumping is entered 
  @button = FindWindowLike( $timer[0], "Set", "" );
  sleep 1;
  # supposedly standard push button method
  PushChildButton( $timer[0], $button[0], 0.25 );
  sleep 1;
  # push button method required for Tcl and also Norhof Monitor
  click_on_middle_of_window( $button[0] );
}
